c    written by Paolo  Coppi 
c    changed by Juri to increase the speed (Oct 28, 1997) 
c    relativistic reflection is added Oct 29, 1997
c    parameters changed by MG on Nov 5, 1997
c    diskbb replaced by diskpn by MG, Jun 28, 1998
c    fits data load routines added by PC, Jul 23, 2001

      SUBROUTINE XSEQTH(Ear,Ne,Param,Ifl,Photar,Photer)
      include "common.eq"
      common /normdisk/ radnorm

      INTEGER Ne , Ifl
      REAL Param(20) , Ear(0:Ne) , Photar(Ne), Photer(Ne)

c
c     driver for the nonthermal/thermal pair program
c
c     number of model parameters: 18
c     parameters are given below in the order of decreasing importance:
c     1: hard-to-soft compactness ratio
c     2: blackbody compactness (if <0, uncomptonized black body component only)
c     3: blackbody temperature in eV (if <0, use diskpn, T_{max}=abs(Par(3)) )
c     4: non-thermal-to-hard compactness ratio (0 for pure thermal plasma)
c     5: Thomson optical depth of ionization electrons (e.g., 0)
c     6: source region radius in cm (for Coulomb/bremsstrahlung only)
c     7: minimum Lorentz factor for power law injection (not used for mono)
c     8: maximum Lorentz factor for power law injection 
c     9: power law injection index (if <0, inject monoenergetically at #8)
c     10 choose primary pair or electron injection (0=electron, 1=pair)
c     11: cosIncl, cosine of inclination angle
c     12, R, amount of reflection (if < 0 then only reflection component)
c     13: FeAb, iron abundance in units of solar
c     14: abundance of elements heavier than He relative to solar
c     15: disk temperature (used for reflection)
c     16: xi, ionization parameter
c     17: beta, reflection emissivity law (r^beta) IF=-10 then 
c         non-rotating disk, IF=10 then  (1.-sqrt(6./rg))/rg**3
c     18: radin, inner radius of the disk in M
c     19: radout, outer radius of the disk in M
c     20: redshift
c
c    Note: if #3 OR #12 < 0, only show BB and/or Reflected components
c
c     algorithm:
c           a(x)=(non-thermal/thermal pair spectrum)+reflection
c

      INTEGER n , np , nem,ne1,neold 
      parameter (nem=5000)
      parameter (np=20,nref=9,nrefl=7,ndisk=2)
      REAL pa0(np),parref(nref),par_refl(nrefl),RedShift,RefScale 
      REAL p1, p4
      DOUBLE PRECISION xg(20),wg(20),wkev(221),
     &     phref(nem),photco(nem),photre(nem),obj(nem),objold(nem),   
     &     photbb(nem)
      common/compsp1/ ospect(221)

      LOGICAL firstcall , fl0 , fl1 , fl2 
      logical showbb

      SAVE firstcall, fl0, fl1, fl2, pa0, parref
      SAVE xg,wg,neold,objold,RedShift,RefScale
      SAVE phref,photre,photco,photbb,fnorm

      DATA firstcall/.TRUE./,fl0/.true./,fl1/.true./,fl2/.true./
      DATA pa0/np*999./,parref/nref*999./,RedShift/0./
      DATA neold/999/ 


c change input XSPEC parameters:
c
c   p(1) = (l_th + l_nth) / l_bb
c   p(2) = l_bb
c   p(4) = l_nth / (l_th + l_nth)
c
c into new values:
c
c   p(1) = l_nth
c   p(2) = l_bb
c   p(4) = l_th

      if (Param(2) .lt. 0.0) then
         Param(2) = -Param(2) 
         showbb  = .true.
      else
         showbb = .false.
      endif

      p1 = Param(1) * Param(2) * Param(4)
      p4 = Param(1) * Param(2) * (1.0 - Param(4))
      Param(1) = p1
      Param(4) = p4

c      print *, 'l_nth = ', p1
c      print *, 'l_th  = ', p4 

c object's frame energies (in mc2)
      ne1=ne+1
      do i=0,ne
         obj(i+1)=dble((1.+Param(20))*ear(i)/511.)
      enddo

      IF ( firstcall ) THEN
         CALL xwrite('     ',5)
         CALL xwrite('            Eqtherm V1.03, FITS data version',5)
         CALL xwrite('Still experimental  -- use at your own risk!',5)

         call gauleg(0.d0,1.d0,xg,wg,10)
      ENDIF


c  calculate internal source spectrum if necessary
cjp checking only for the first 10 parameters (forget about accuracy)
      DO 100 n = 1 , 10 
         IF ( Param(n).NE.pa0(n) ) fl0 = .TRUE.
 100  CONTINUE
c Check also R_in
      if (Param(18) .ne. pa0(18)) fl0 = .TRUE.
c      if (fl0) print *,'need to recompute model'
      IF (fl0.or.Param(20).ne.RedShift) THEN
         CALL thpairiter(Param,firstcall,np)
         do i=1,221
c total emergent spectrum including black body
            esc(i) = esc(i)/gwidth(i)
c separate Comptonized continuum and bb!!!! 
c
            bbspect(i) = bbspect(i)/gwidth(i)
            ospect(i)=esc(i)-bbspect(i)
c ACHTUNG
c    can't allow zeros or things get all messed up 
c    (spect interp. takes log(0)-- oops)
            ospect(i) = max(1.d-30,ospect(i))
            bbspect(i) = max(1.d-30,bbspect(i))
            tspect(i) = max(1.d-30,tspect(i))
         enddo
c screws up near annihilation line!
c          call qspline(wkev,ospect,221,1d30,1d30,f2)

c renormalize to disk model
         fnorm = radnorm / 511.0

c      xn = (1.d0+Param(20))/511. 
c interpolate to get spectrum at 1 KeV
c        targ = xn
c        i = max(min(int(log10(targ)*20.d0+141.5d0),221),1)
c        if (targ.lt.wgam(i))  i=max(i-1,1)
c        i2 = min(221,i+1)
c        x0 = wgam(i)
c        x1 = wgam(i2)
c        y1 = ospect(i2)
c        y0 = ospect(i)
c        if (i.ne.i2) then 
c           tempeqp = y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
c        else
c           tempeqp = y0
c        endif
cjp normalizing only continuum radiation without reflection 
c        fnorm = 1.d0/tempeqp

         do i=1,221
            ospect(i) = ospect(i)*fnorm
         enddo

      ENDIF
 


cc ** Now add reflection!
c      
c check obj 
      if(neold.ne.ne) fl2=.true.
      do i=1,ne1 
         if(obj(i).ne.objold(i)) fl2=.true.
      enddo 
c check if reflection parameters have changed 
      do i=13,19 
         if(param(i).ne.parref(i-10)) then 
            fl1=.true. 
         endif 
      enddo 
      if(param(11).ne.parref(1)) fl1=.true.
      if(RedShift.ne.Param(20)) fl1=.true.

c reflection 
c normalization 
      RefScale=abs(Param(12)) 
      IF((fl0.or.fl1.or.fl2).and.(RefScale.gt.1e-6.or.firstcall)) THEN 
cc ionized reflector (and possibly rotating) 
c  ab0, iron log10 abundance relative to solar
         par_refl(1)=alog10(Param(13)) 
c  ab_, log10 abundance of elements heavier than He relative to
c       the solar abundances
         par_refl(2)=alog10(Param(14))
c  xi, ionization parameter
         par_refl(3)=Param(16)
c     temp, disk temperature (for ionization balance, Tmax=10^6 K)
         par_refl(4)=Param(15)
c  beta      - reflection emissivity law (r^beta)
         par_refl(5)=Param(17)
c  radin     - inner radius of the disk
         par_refl(6)=Param(18)
c  radout    - outer radius of the disk
         par_refl(7)=Param(19)
c cos angle 
         angle = Param(11)
c     wgam - energies (221)
c   ospect  - angle averaged comptonized spectrum 
cjp ACHTUNG: here one should have ONLY comptonized spectrum
cjp NOT bb+comptonized !!!!!!!! 
c     obj - energies where refl should be computed
c     phref  - reflected spectrum at a given angle
         irefl=0
c relativistic disk reflector
         if(Param(17).ne.-10.) irefl=1 
         call comprefl1(wgam,ospect,obj,phref,angle,
     +        par_refl,nrefl,221,ne1,irefl)
      ENDIF 

c    rebinning of the comptonized spectrum
      IF(fl0.or.fl2.or.Param(20).ne.RedShift) THEN 
         do i=1,221
            wkev(i) = wgam(i)*511d0
         enddo
         do it=0,Ne-1
            xlo = Ear(it)
            xhi = Ear(it+1)
            dx = xhi-xlo
            sum = 0.d0
            do j=1,10
               targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c--- inline expansion
               if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
                  tempeqp = 1d-30
               else
                  i = max(min(int(log10(targ/511.d0)*20.d0+
     &                 141.5d0),221),1)
                  if (targ.lt.wkev(i))  i=max(i-1,1)
                  i2 = min(221,i+1)
                  x0 = wkev(i)
                  x1 = wkev(i2)
                  y1 = ospect(i2)
                  y0 = ospect(i)
                  if (i.ne.i2) then 
                     tempeqp= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
                  else
                     tempeqp = y0
                  endif
               endif
               sum = sum + tempeqp*wg(j)*dx
            enddo
            photco(it+1) = sum
         enddo

c    rebinning of the bb spectrum
         do it=0,Ne-1
            xlo = Ear(it)
            xhi = Ear(it+1)
            dx = xhi-xlo
            sum = 0.d0
            do j=1,10
               targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c---  inline expansion
               if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
c this must be tempbb - kaa
c                  tempeqp = 1d-30
                  tempbb = 1d-30
               else
                  i = max(min(int(log10(targ/511.d0)*20.d0+
     &                 141.5d0),221),1)
                  if (targ.lt.wkev(i))  i=max(i-1,1)
                  i2 = min(221,i+1)
                  x0 = wkev(i)
                  x1 = wkev(i2)
                  y1 = bbspect(i2)
                  y0 = bbspect(i)
                  if (i.ne.i2) then 
                     tempbb= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
                  else
                     tempbb = y0
                  endif
               endif
               sum = sum + tempbb*wg(j)*dx
            enddo
c ** DON'T FORGET normalization
            sum = sum * fnorm
            photbb(it+1) = sum
         enddo
      ENDIF 

c -    now get reflection component
      IF(RefScale.gt.1e-6) THEN
         IF(fl2.or.fl1.or.fl0.or.Param(12).ne.parref(2)) THEN 
c integrate over the bin size
            do i=1,Ne
               photre(i)=(phref(i+1)+phref(i))*
     &              dble((ear(i)-ear(i-1))/2.)*RefScale
            enddo
         ENDIF 
      ELSE 
         do i=1,Ne
            photre(i)=0. 
         enddo
      ENDIF 


      do i=1,Ne
         photar(i) = 0.0
         if ((Param(12).lt.0.).or.(showbb)) then
            if (Param(12).lt.0.) photar(i)=photre(i)+photar(i)
            if (showbb)          photar(i)=photbb(i)+photar(i)
         else
            photar(i)=sngl(photco(i)+photre(i)+photbb(i))
         endif
      enddo

      Do n = 1 , np
         pa0(n) = Param(n)
      EndDo
      fl0 = .FALSE.

      Do n = 11 , 19
         parref(n-10) = Param(n)
      EndDo
      RedShift=Param(20) 
      fl1 = .FALSE.

      Do i = 1, Ne1 
         objold(i)=obj(i)
      EndDo
      neold=ne 
      fl2 = .FALSE.

      firstcall = .FALSE.
c      print *,'o.k..finished w/eqpair'

      RETURN
      END

c ===================================


      subroutine thpairiter(param,firstcall,np)

      include "common.eq"

      real param(np)
      logical firstcall


c set model parameters and initialize model
      call thsetup(param,firstcall,np)

c double check for bad input parameters
      if((rle .lt. 1.d-40).and.(rlth .lt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
c        do nothing!!  (no pairs!, gdist(i) = s(i) in setup)
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif
      if((rle .lt. 1.d-40).and.(rlth .gt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
         print *,'ILLEGAL INPUT COMBINATION:  l_th is not zero, yet you'
         print *,'have no background electrons/positrons since tau_p=0'
         print *,'and l_nth=0!!     '
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif


c get model spectrum
      call thsolve

c get uncomptonized blackbody spectrum

      call bbesc

      return
      end

c ---------------------------
      subroutine thdopcomp
      include "common.eq" 
      data thresh /0.1/

      if ((diff .lt. 0.3d0).and.(niter.gt.5)) then 
         thresh = 0.1d0
      else
         thresh = 2.d0
      endif
c      print *,'thresh',thresh,diff

       iphot = 0

c assume pout cleared, pin set to s(i)
c assume e- dist fixed


c call doesc + annihilation + brem + anything else that makes photons
       do i=1,ngambin
          gder(i) = 0.d0
       enddo
       call thdotherm



       do i=1,ngambin
          pout(i) = pout(i) + tesc(i)
          pin(i) = pin(i) + gder(i)
          pin0(i) = pin(i)
       enddo



       if (itherm.eq.1) then
          call mkscat
       endif


       do i=1,ngambin
          pout0(i) = pout(i)
       enddo

c  now iterate


 1     continue




       if (itherm.eq.1) then
          do i=1,ngambin-20
             do j=iscat(i,1),iscat(i,2)
                pin(j) = pin(j) + scat(i,j)*gdist(i)
             enddo
          enddo
       endif


       dmax = 0
       do i=1,ngambin
          temp = pin(i)/pout(i)
          odmax = dmax
          dmax = max(abs(temp-gdist(i))/(gdist(i)+1.d-40),dmax)
          if (dmax.ne.odmax) idmax = i
          gdist(i) = temp
       enddo


       if (dmax .gt. thresh) Then
          do i=1,ngambin
             pout(i) = pout0(i)
             pin(i) = pin0(i)
          enddo
          goto 1
       endif


       do i=1,ngambin
          esc(i) = tesc(i)*gdist(i)
       enddo


       return
       end

c =========================================================
c dump out particle distributions

      subroutine thinfo
      include "common.eq"

c      backspace(1)
      nz = nz + 1
      write(1,*)'------------------------------------------------'
      write(1,*)'''ITERATION# ',iter,' le =',rle,' ls =',rls,''''
      write(1,*) '''rlth =',rlth,'taup=',taup,''''
      write(1,*)'''-- iterations since last report: ',iter-niter,''''
      write(1,*) '''Locomp , sync '''
      write(1,*) ians1,ians2
      write(1,*) '''B0,  Source Rad(m.) '''
      write(1,*) b0,rad
      noiter = iter
      write(1,*)'''Time = '''
      write(1,*) time
      etot = 0.
      trnpart = 0.
      do 30 i = 2,nebin
         etot = etot + een(i)*(edist(i)+posdist(i))
         trnpart = trnpart+(edist(i)+posdist(i))
 30   continue
      etot = etot+ 2.*eth
      trnpart = trnpart + (edist(1)+posdist(1))
      enesc = 0.
      do 31 i = 1,ngambin
         etot = etot + wgam(i)*gdist(i)
         trnpart = trnpart+gdist(i)
         enesc = geoz * esc(i)*wgam(i) + enesc
c     print *,i,esc(i),gdist(i)
 31   continue
      write(1,*) '''theta = '''
      write(1,*) theta,xs
      xdiff = abs(trnpart - rnpart)
      rnpart = trnpart
      write(6,*) 'Etotal = ',etot
      write(6,*) 'rnpart = ',rnpart
      write(6,*) 'idump = ',idump
      write(6,*) ' dt = ',dt
      write(1,*)'''Etotal = ''',etot
      write(1,*)'''Total # of particles:''',rnpart
      write(1,*)
      write(1,*) '''rate cascading into bin 2: ''',deltan
      write(1,*) '''minimum timestep in e- bin ''',idtmin,' = ',dt
      write(1,*)'''e- bin values: (Gam,min,max) ''',plawin,igmin,igmax
      do 10 i = 1,nebin
c     write(1,*)een(i),edist(i),eder(i),gnder(i),gcder(i)
         write(1,299)een(i),edist(i),posdist(i),gcder(i)
c299   format(2x,g12.6,4x,g13.7,4x,g13.7)
 299     format(2x,g12.6,4x,g13.7,4x,g13.7,4x,g13.7)
 10   continue
      write(1,*)
      write(1,*)'''photon bin values:'''
      sum = 0.
      do 20 i = 1,ngambin
         sum = sum + wgam(i)*esc(i)*geoz
         write(1,399) wgam(i),geoz*s(i),geoz*esc(i),sum
 20   continue
 399  format(2x,g12.6,4x,g13.7,4x,g13.7,4x,g13.7)
      write (1,*) '''Escaping photon luminosity: ''',enesc
      write (1,*) '''Escaping pair   luminosity: ''',.3*eth*2.*geoz
      print *,nz
      write (1,*) 'Pair yield: ',py ,nz
c -- For Piero
c      write(1,*) '''Tau_comp(x): '''
c      do 88 i = 1,ngambin
c        tau = 0.
c        do 89 j = 2,nebin
c          tau = tau+2.*ecomp(j,i,1)*edist(j)
c89      continue
c        write(1,499) wgam(i),tau
c499     format(2x,g12.6,4x,g13.7)
c88    continue
c      endfile(1)
      call flush(1)
c        do i=1,ngambin
c          gder(i) = 0.
c        enddo
c        call dotherm
c        
c        do i=1,ngambin
c        write(2,*) wgam(i),geoz*gder(i)*tesc(i)
c        enddo
      return
      end

c============================================

      subroutine thgetth
      include "common.eq" 
      external dtime
      real dtime
      real time(2)
      logical iboink

c  ------   Get Optical Depth in equilibrium
c for pair inject
      if (pairinj) then
         ein(1) = psum + erninj
         posin(1) = psum + erninj
      else
         ein(1) = psum
         posin(1) = psum
      endif

c new experiment -- let's compute tau dynamically again and see what 
c happens
c      if (iread.gt.0) then
      call thgetethd
      edist(1) = taup
      posdist(1) = 0.d0
c        print *,'NEW TAU : ',posdist(1) + edist(1),n
      
 
      call thdotherm
   
      do i=1,ngambin
         gdist(i) = gdist(i)*otesc(i)/tesc(i)
c muy important 
         otesc(i)  = tesc(i)
      enddo
c      endif


      realotheta = theta

      if (itherm.gt.0) then

         ith= 0
         dnth = 2.d0*psum
c pair injection
         if (pairinj) then 
            ethder0 = dnth*een(1) + direthder + 2.d0*erninj*een(1)
         else
c electron injection
            ethder0 = dnth*een(1) + direthder
     &           +(een(1)-1.d0)*(erninj+prninj)
         endif

c try a trial temperature and figure out which the gradient
c of ethder is pointing
         temp = theta
         if (ethder .gt. 0.) then
            theta = theta*1.001
         else
            theta = theta*.999
         endif
         ethder = ethder0
         call thgetethd
c ** don't forget coulomb heating & brems
         ctemp = ethder
         oethder = ethder
         otheta = theta
         theta = temp
         yeoldth = otheta
         iboink = .false.
         dn = 1.0 
 336     continue
         ethder = ethder0
         call thgetethd

         if (ethder.eq.oethder) then
c     print *,'temp ethder mucked!'
         endif
         if (theta.ne.otheta) then
            drdth=(ethder-oethder)/(theta - otheta)
         else
            drdth = 0.0
            print *,'theta = otheta   this should not happen'
         endif
c           print *,'theta,otheta,ethder,oethder,drdth',theta,otheta,
c     &         ethder,oethder,drdth
         otheta = theta
         oethder = ethder
         if(drdth.ne.0) theta = theta - ethder/(drdth)*dn
         if (theta.lt.1.d-4) then
            dn = (otheta-1.d-4)/ethder*drdth
            theta = otheta
            theta = 0.5*otheta
c     print *,'theta boink!... ouch'
            iboink  = .true.
         endif
         if (theta.gt.50.d0) then
            theta = min(50.d0,1.07*otheta)
c     print *,'theta bink!...eech'
         endif
         dth = abs(theta - otheta)/(abs(theta)+1.d-21)
c     print *,'th,oth,thd,othd',theta,otheta,ethder,oethder


         ith = ith + 1
         if (ith.gt.15) then
            print *,'ith ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZAP'
            if (iboink) then
               theta = max(0.8*theta,1.d-4)
            else
               theta = realotheta*1.2
            endif
            goto 889
         endif

         if (abs(dth) .lt. 1.d-6) goto 887
         goto 336

 887     continue

 889     continue

         dn = min(.5d0,1.d0/(1.d0+3.d0*realotheta))
         theta = dn*theta + (1.d0-dn)*realotheta
      endif
   
c       print *,'final theta = ',theta

      return
      end


c=====================

      subroutine thsetup(param,firstcall,np)
      include "common.eq"
      common /normdisk/ radnorm

      logical firstcall
      real param(np), tparam(2),tphotar(221),tear(0:221)

c      print *,'in setup' 
c      rle = param(1)
      rle = 0.d0
      rls = param(2)
c blackbody temperature in m_ec^2
      tr  = param(3)/5.11d5
      rlth = param(4)
      taup = param(5)
      rad  = param(6)

      Rin = param(18)

      igmax = max(min(int(log10(param(8))/.05d0+1.5d0),81),2)
      plawin = param(9)
      if (plawin .gt. 0.d0) then
         igmin = max(min(int(log10(param(7))/.05d0+1.5d0),81),2)
      endif
      if (param(10).gt.0) then
         pairinj = .true.
      else
         pairinj = .false.
      endif
      ians2 = 0
      b0 = 0.d0
      ians1 = 1
cJP is it OK? it should be param(18) but in fact it is always 0.02
      econv = param(18)
c      econv = param(16)

      maxiter = 10000

c      print *,'db: rle,rls,tr,rad',rle,rls,tr,rad
c      print *,'igmin,igmax,plawin',igmin,igmax,plawin
c      print *,'econv,maxiter',econv,maxiter

      nebin = 81
      ngambin = 221
      ioff = 40
      rioff1 = 141.5
      wmin = 10**(-7.025)
      wmax = 10.**4.025
c initial idump = bin# e- cascade into when using stationary approx
      idump = nebin
c initialize energy bin values
      do 58 i = 1,nebin
         temp = 10.**(float(i-1)*.05d0)
         een(i) = .5d0*temp*(10.**.025d0 + 10.**(-.025d0))
         upeen(i) = 10.**.025d0*temp
 58   continue
      een(1) = (10.**.05d0 - 1.)/2./(10.**.025d0 - 1.)
      do 59 i = 2,nebin
         ewidth(i) = upeen(i) - upeen(i-1)
 59   continue
      ewidth(1) = upeen(1) - 1.d0
c do photons
      do 62 i = 1,ngambin
         temp1 = 10.**(float(i-101-ioff)*.05d0)
         wgam(i) = .5d0*temp1*(10.**.025d0 + 10.**(-.025d0))
         upwgam(i)= 10.**.025d0*temp1
c        print *,i,wgam(i)
c        pause
 62   continue
      do 60 i = 2,ngambin
         gwidth(i) = upwgam(i) - upwgam(i-1)
 60   continue
      gwidth(1) = upwgam(1)-10.**(-7.025d0)
c
c  --- get model parameters
c      write(6,*) 'Hello...'
c      write(6,*) 'Model parameters are: '
c      write(6,*) 'tr = ',tr
c define source (injection) functions
c q(i) - PAIR inject funct, s(i) - photon inject. func.
c power law(?) injection
      do 45 i = 1,nebin
         q(I) = 0.
         pq(i) = 0.
 45   continue

c Take care of injection:
c ***
c *** ACHTUNG: there is a geometric factor relating le/s to
c       q,s the >dimensionless<  particle injection rates:
      geoz = 16.*atan(1.)/3.
c    (this ^ is for comparison with big Z,FGBC geoz = 1.)

      urad= 0.0
      ue = 0.0
c *factor of -1 is to agree with big Z *
      if (rle .ne. 0.) then
         if (plawin .lt. 0.) then
            write(6,*) '** Monoergetic injection at gamma =',een(igmax)
            q(igmax) = 1.
         else
c          write(6,*) '** Power law injection with index: ',plawin
c          write(6,*) '   from gmin =',een(igmin),'to gmax =',een(igmax)
            do 422 i = igmin,igmax
               q(i) = een(i)**(-plawin)*ewidth(i)
 422        continue
         endif
         rntot = 0.
         do 47 i = 1,nebin
            rntot = rntot + q(i)
            ue = q(i)*(een(i)) + ue
 47      continue
c remove rest mass since it is reaccelerated away
c if have pair injection then things more complicated because
c of pair annihilation -- in practice we won't bother with it
         if (pairinj) then 
c           print *,'PAIR INJECTION'
            ue = 2*ue
         else
            ue = ue - rntot
c           print *,'ELECTRON INJECTION'
         endif


         do 67 i = 1,nebin
            q(i) = q(i)/ue*rle / geoz
 67      continue
         ue = 0.
         do 68 i = 1,nebin
            ue = 2.*q(i)*(een(i))*geoz + ue
 68      continue
      endif
c  black body photon injection at xs
      if (rls .ne. 0.) then
         if (tr .lt. 0.) then 
c         use diskpn model
            tparam(1) = -tr*511.0
            tparam(2) = Rin
            do i=1,ngambin
               tear(i) = upwgam(i)*511.0
            enddo
            tear(0) = upwgam(1)*10.**(-0.05d0)*511.0
            call xsdiskpn(tear,221,tparam,1,tphotar)
            do i=1,ngambin
               s(i) = tphotar(i)
            enddo
c        ^^ may be a constant of 511 floating around, but gets normalized out
c           I think
         else 
c         use standard blackbody
            do 66 i = 1,ngambin
               tbb = 1.e18*gwidth(i)*wgam(i)*wgam(i)
               texp = wgam(i)/tr
               texp = .5*(texp-1.+sign(texp+1.,60.-texp))
               tbb = tbb/(exp(texp)-1.)
               s(i) = dmax1(0.d0,tbb)
 66         continue
         endif
         do 48 i = 1,ngambin
            urad = s(i)*wgam(i)+ urad
 48      continue
         do 49 i = 1,ngambin
            s(i) = s(i)/urad*rls/geoz
 49      continue

c Normalization of disk component, will be used to renormalize total spectrum
c to make the XSPEC norm of the total spectrum equal norm of the disc spectrum.
c Thus, the inner disk radius can be computed.

         radnorm = urad * geoz / rls

         urad = 0.0
         do 50 i = 1,ngambin
            urad = s(i)*wgam(i)*geoz+ urad
50       continue
      else
        do 102 i = 1,ngambin
           s(i) = 0.
 102    continue
      endif

c      write(6,*) 'urad-dot: ',urad
c      write(6,*) 'ue-dot: ',ue
c      write(6,*) 'q(61) = ',q(61)

c      if don't read from file, put
c        initial non-zero starting distribution into bins
      do 87 i = 1,nebin
         edist(i) = 1.e-7
 87   continue
      eth = edist(1)*een(1)
c fudge --
      theta = 2./3.*(eth/edist(1)-1.)

c  initialize tables
c      if (ians1 .ne. 0)
c     &   write(6,*) 'Extended low photon energy range enabled.'
c      if (ians2 .ne. 0) then
c        write(6,*) 'Synchrotron radiation enabled.'
c        write(6,*) ' --Magnetic field strength: ',b0
c        write(6,*) ' --Source region radius: ',rad
c      endif
      if (firstcall) then 
         call load
c     write(6,*) 'file tables loaded in...?'
      endif


c  ** Initial guesses for various parameters
      do 766 i = 1,nebin
         edist(i) = 0.
 766  continue
      rinject = 0.
      do 234 i=1,nebin
         rinject=rinject+q(i)
 234  continue
        
      do i=1,ngambin
         gdist(i) = s(i)
      enddo

c      write(6,*) 'o.k. here goes nothing...'
      return
      end

      subroutine thsolve
      include "common.eq"
      dimension pstore(241)

c -- set up constants
      dlg = log(een(5))-log(een(4))
      do 876 i=2,nebin
         deltag(i) = een(i)-een(i-1)
876   continue
      tenesc = (rle+rls+rlth)/geoz

      opy = 0.
      itherm = 0
      nphot = 0
c      theta = .6*(een(1)-1.0)
      u = een(1)
      a  = 45./4.
      b = (27.-15.*u)/8.
      c= 1. - u
      theta0 = (-b+sqrt(b*b-2.*a*c))/a

      edist(1) = 0.d0 + taup
      posdist(1) = 0.d0
c      print *,'>> initial guess for tau:',edist(1)

c * initial coulomb cooling rate
      cturnon = 1.d0

      erninj = 0.
      prninj = 0.

c      print *,'thermal injection compactness', rlth
      direthder = rlth/geoz


c      print *,'maximum number of iterations: '
c      read *,niter
c      print *,'starting iterations...'
      idump = 1

c iteration difference to stop at before start thermal
      tdiff1 = 1.d0
c iterate until escaping energy has converged to better than econv
      econv = 2d-2

c  --- start main loop
c  --- (assume theta, eth, edist(1) initialized in setup! Ja?)
c  aber nicht otheta
      otheta = theta
      oetotal = 0

      niter = 0
      do i=1,ngambin
         otesc(i) = 1.
         pstore(i) = 0.
      enddo
      psum = 0

1     continue
      niter = niter + 1

      do 20 i = 1,ngambin
         esc(i) = 0.
         pout(i) = 0.
         pin(i) = s(i)
 20   continue
      ethder = 0.
      do 21 i=1,nebin
         gdot(i) = 0.
         ein(i) = q(i)
c for pair injection
         if (pairinj) then 
            posin(i) = q(i)
c for electron and/or positron injection
         else
            posin(i) = pq(i)
         endif
         eout(i) = 0.
         posout(i) = 0.
         eout2(i) =0.
         posout2(i) = 0.
 21   continue
      pause = 0

c     get electron distribution
      call thdoelec
c   -- now do photon distribution
      call thdopcomp

      do i=1,ngambin
         otesc(i) = tesc(i)
      enddo


c ---- Overall convergence check (look at photon distribution)
      diff = 0.
      temp = 0.0
      do 133 i = 1,ngambin
c change for linux
         if (gdist(i).gt.1.d-38) temp=abs(pstore(i)-gdist(i))/gdist(i)
         if (temp.gt.diff) then 
            id = i
            diff = temp
         endif
 133  continue

c don't forget to update pstore
      do 132 i =1,ngambin
         pstore(i) = gdist(i)
 132  continue

c      print *,'DIFF = ',diff,theta,edist(1),id
c      print *,'tau = ',edist(1)+posdist(1),'py = ',psum
c      print *,'niter',niter

      enesc = 0.0
      do i=1,ngambin
         enesc = enesc + esc(i)*wgam(i)
      enddo
c      print *,'energy escaping',enesc,'vs. expected',tenesc

      if ((diff .lt. tdiff1).or.
     &     (nphot .gt. maxiter)) then
c         print *,'o.k. passed diff test'
         if (nphot .gt. maxiter) then 
            print *,' >> Run ended prematurely: max iter hit'
            return
         endif
         ediff = abs(enesc-tenesc)/(tenesc+1.d-12)
         if((ediff .lt. econv).and.(itherm.gt.0)) then
c          print *,'convoiged!!'
c          print *,'*** theta = ',theta*511.d0,' kev',
c     &            ' tau_T = ',edist(1)+posdist(1)
c          open(unit=1,file='final',status='unknown',form='formatted')
c          call info
c          close(1)
            return
         endif
         if (itherm .eq. 0) then
            itherm= 1
            tdiff1=1.d-2
            goto 1
         endif
      endif
      
c -- go back for another round of convergence talks
      goto 1     

      end 



c ================================

      subroutine thdotherm
      include "common.eq"


c -- START: check temperature first
c      if (theta .gt. .15) write(6,*) 'Warning theta too high!',theta
c      if (theta .lt. .001) write(6,*) 'Warning theta too low!',theta

      x = log10(theta)
c*********      IDIOTSKI DON'T FORGET
      eth=(1.d0+27.d0/8.d0*theta+45.d0/8.d0*theta*theta)
     &   /(1.d0+15.d0/8.d0*theta)*(edist(1)+posdist(1))

c --- annihilation
c   (first do annihilation of thermal bin with itself)
c use Svennson approx.
c
c zero out gder(i) so we can still thscat routine verbatim
      do 233 i=1,ngambin
         gder(i) = 0.
233   continue
  

      esum = 0
      do i=1,ngambin
         esum = esum + gder(i)*wgam(i)
      enddo


c ---- PAIR Escape!!!
c      (use AZ phenomenological fudge)
c      eder(1) = eder(1) - .3*edist(1)
c      ethder = ethder - .3*eth
c      eout(1) = eout(1) + .3

c ** initialize escape array
c ** Remember N for Tau_thomp = (ne- + ne+)=2!!!*edist(1)
      tau = 1./3.*(edist(1)+posdist(1))
      do 303 i = 1,ioff
         tesc(i) = 1.d0/(1.d0+tau)
 303  continue
      do 30 i = 1+ioff,81+ioff
         ii = i-ioff
         if (theta .lt. .15) then
            trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
            x1 = log10(.15)
            x2 = log10(.16)
            trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
            qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
            qrate=10.**qrate
            trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
            trate=dmin1(1.d0,trate)
c         elseif (theta .ge. .16d0) then
         else
            trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
            trate=10.**trate
            trate=dmin1(trate,1.d0)
         endif
         tau = 1./3.*trate*(edist(1)+posdist(1))
         tesc(i) = 1./(1.+tau)
 30   continue
      do 309 i =82+ioff,100+ioff
         ii = i - ioff
         if (theta .lt. .15) then
            trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
            x1 = log10(.15)
            x2 = log10(.16)
            trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
            qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
            qrate=10.**qrate
            trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
            trate=dmin1(1.d0,trate)
c         elseif (theta .ge. .16) then
         else
            trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
            trate=10.**trate
            trate=dmin1(trate,1.d0)
         endif
         tau = 1./2.7*trate*(edist(1)+posdist(1))*(1.-wgam(i))
         tesc(i) = 1./(1.+tau)
 309  continue
      do 399 i =101+ioff,181+ioff
         tesc(i)= 1.d0
 399  continue
      return
      end


      subroutine thgetethd
      include "common.eq"
      dimension wi(15),a(6),b(6),ien(15),xt(6)
c * xi values
      data  a /0.222846604179,
     *     1.188932101673,
     *     2.992736326059,
     *     5.775143569105,
     *     9.837467418383,
     *     15.982873980602/
c * wi values   
      data b /4.58964673950e-1,
     *     4.17000830772e-1,
     *     1.13373382074e-1,
     *     1.03991974531e-2,
     *     2.61017202815e-4,
     *     8.98547906430e-7/

      x = log10(theta)


c*********  IDIOTSKI DON'T FORGET
      eth=(1.d0+27.d0/8.d0*theta+45.d0/8.d0*theta*theta)
     &   /(1.d0+15.d0/8.d0*theta)*(edist(1)+posdist(1))

      tethder = ethder



c
c --- compton scattering
c(Assume e- aren't scattered out of thermal dist.)
c --- Also do photon escape (only count thermal e+/e- in
c      determining escape rate. Escape rate formula is one
c      used by big Z.)

c *** Use SLEDGE hammer approach for everything, ie. the laguerre
c   integration fudge. (may be a bit of overkill, but we must get
c   rid of glitches und have Ordnung hier!)
c*** sledge hammer approach
      thint = 0.0
      do 189 lm = 1,6
         xp = a(lm)*theta + 1.0
         xt(lm) = xp
         if (xp .ge. 101.) then 
            ien(lm) = max(int(log10(xp)/.05+1.5),1)
         else
            ien(lm)=-max(int(log10(xp-1.0)/.05+101),1)
         endif
         wi(lm) = b(lm)*xp*sqrt(xp*xp-1.0)
         thint = thint + wi(lm)
 189  continue
      do 190 lm=1,6
         wi(lm)=wi(lm)/thint*(edist(1)+posdist(1))
 190  continue
      do 310 ii = 1,6
         i = ien(ii)     
         weight = wi(ii)
         if (i .gt. 0) then 
c ====     use non-thermal comp. data
c ignore photons > x=10^3
            do 320 j = 1,ngambin-ioff-20
               jj = j + ioff
               grate = ecomp(i,j,1)*weight*gdist(jj)
               ethder = ethder + (wgam(jj)-av(i,jj))*grate
 320        continue
c  go do low energy comp. the same way
c  NB: All rates are assumed to be=sigma_t, since should be in
c       Thompson limit.
c  *** 80 should be ibot !!!! (in program with lcinit)
c achtung, yup, but knocked off two decades
            do 420 j = 1,40
               rate = weight*gdist(j)
c surely this should be rate not grate ? - kaa
c               ethder = ethder + (wgam(j)-clav(i,j))*grate
               ethder = ethder + (wgam(j)-clav(i,j))*rate
 420        continue
c       ** (end ians1 if, for locomp)
c == end of non-therm. data use
         else
c == use extended, low gamma data
            i = -i
            ip1 = min(i+1,141)
            if (i .eq. ip1) then
               frac = 0
            else
c BUGUGUGU!!!!!!!!!!!!!!!!!!!!!!!!!????????
c         frac=(xt(ii)-exn(ip1))/(exn(ip1)-exn(i))
c YES, idiotski!!! DOUBLE CHECK
               frac=(xt(ii)-exn(i))/(exn(ip1)-exn(i))
            endif
            do 321 j=1,ngambin -20
               jc = j + 40
               trate=cltr(i,jc)+frac*(cltr(ip1,jc)-cltr(i,jc))
               eav  =clta(i,jc)+frac*(clta(ip1,jc)-clta(i,jc))
               grate = trate*weight*gdist(j)
               ethder = ethder + (wgam(j)-eav)*grate
 321        continue
         endif
 310  continue

      return
      end

c============================
      subroutine thdoelec
      include "common.eq" 
      double precision eout0(81),posout0(81)
      external dtime
      real dtime
      real time(2)

c      start = dtime(time)

c      goto 232
c get sources
c  -----------  pair production

      superoth = theta
      isth = 0

       

c get thomson optical depth
      call thgetth



      do i=1,1
         ein0(i) = ein(i)
         posin0(i) = posin(i)
         eout0(i) = eout(i)
         posout0(i) = posout(i)
      enddo

1     continue

      do i=1,1
         ein(i) = ein0(i)
         posin(i) = posin0(i)
      enddo
 
      dmax = 0


      call thgetth
      dth = abs(theta-superoth)/superoth
      superoth = theta
c      print *,'.....................dth',dth

      isth=isth + 1
      if ((dth .lt. 1.d-2)) goto 928
      do i=1,1
         ein(i) = ein0(i)
         posin(i) = posin0(i)
         eout(i) = eout0(i)
         posout(i) = posout0(i)
      enddo

      goto 1

 928  continue

c      stop = dtime(time)
c      print *,'e- time=',time(1)

      return
      end
